// ChatDlg.cpp : implementation file
//

#include "stdafx.h"
#include "zChatClient.h"
#include "ChatDlg.h"
#include "zTcpSock.h"
#include "ConnectionDlg.h"

#include "..\plugin API\ExtAPI.h"
extern CExtAPI theAPI;

#include "..\plugin API\zFunc.h"
#include "..\plugin API\zVar.h"
#include "..\plugin API\zStr.h"
#include "..\plugin API\zList.h"
#include "..\plugin API\zBuffer.h"



// CChatDlg dialog

IMPLEMENT_DYNAMIC(CChatDlg, CDialog)
CChatDlg::CChatDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CChatDlg::IDD, pParent)
	, m_RecvText(_T(""))
	, m_SendText(_T(""))
{
	m_IsShown = false;
	m_IsCreated = false;

	m_SockMode = 0;
	m_PeerAddr = _T("127.0.0.1");
	m_UserName = _T("");
	m_PortNo = 8080;
}

CChatDlg::~CChatDlg() {
	HideWnd();
}

void CChatDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CONNECT_BUTTON, m_ConnectBtnCtrl);
	DDX_Text(pDX, IDC_RECV_EDIT, m_RecvText);
	DDX_Text(pDX, IDC_SEND_EDIT, m_SendText);
	DDX_Control(pDX, IDC_SEND_BUTTON, m_SendBtnCtrl);
	DDX_Control(pDX, IDC_SEND_EDIT, m_SendEditCtrl);
	DDX_Control(pDX, IDC_RECV_EDIT, m_RecvEditCtrl);
}


BEGIN_MESSAGE_MAP(CChatDlg, CDialog)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_CONNECT_BUTTON, OnBnClickedConnectButton)
	ON_BN_CLICKED(IDC_HELP_BUTTON, OnBnClickedHelpButton)
	ON_BN_CLICKED(IDC_SEND_BUTTON, OnBnClickedSendButton)
	ON_WM_TIMER()
END_MESSAGE_MAP()


// CChatDlg message handlers

BOOL CChatDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// get icon
	CzFunc f;
	HICON hIcon;
	if(f.GetFunc("application.GetIcon")) {
		if(theAPI.var_GetValueEx(f.Execute(), VarID_LPVOID, &hIcon)) {
			SetIcon(hIcon, true);
		}
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}
bool CChatDlg::IsShown() { return m_IsShown; }
void CChatDlg::ShowWnd() {
	if(m_IsShown)
		return;

	if(!m_IsCreated) {
		CDialog::Create(IDD_CHAT_DIALOG, NULL);
		m_IsCreated = true;
	}

	ShowWindow(SW_SHOWNORMAL);
    m_IsShown = true;

	FormatControls();
}

void CChatDlg::HideWnd() {
	if(!m_IsShown)
		return;

	CloseAll();

	ShowWindow(SW_HIDE);

	m_IsShown = false;
}

void CChatDlg::OnClose() {

	HideWnd();
	
	CDialog::OnClose();
}

void CChatDlg::OnBnClickedOk() {
	
	HideWnd();

	OnOK();
}

void CChatDlg::OnBnClickedConnectButton() {

	UpdateData(TRUE);

	bool DoOpenSocket = false;
	if(m_SockMode==0) { // client
		if(!m_CliSock.GetSockHandle()) {
			DoOpenSocket = true;
		} else {
			int SockState = m_CliSock.GetSockState();
			if(SockState==SOCKSTATE_CONNECTED) {
				m_CliSock.Close();
			} else
			if(SockState==SOCKSTATE_CLOSED) {
				DoOpenSocket = true;
			}
		}
	} else
	if(m_SockMode==1) { // server
		if(!m_ListenSock.GetSockHandle()) {
			DoOpenSocket = true;
		} else {
			int SockState = m_ListenSock.GetSockState();
			if(SockState==SOCKSTATE_LISTENING) {
				m_ListenSock.Close();
				CloseClients();
			} else
			if(SockState==SOCKSTATE_CLOSED) {
				DoOpenSocket = true;
			}
		}
	}

	if(DoOpenSocket) {
		CConnectionDlg dlg;
		dlg.m_SockMode = m_SockMode;
		dlg.m_PeerAddr.SetString(m_PeerAddr);
		dlg.m_PortNo = m_PortNo;
		dlg.m_UserName.SetString(m_UserName);
		if(dlg.DoModal()==IDOK) {
			m_SockMode = dlg.m_SockMode;
			m_PeerAddr.SetString(dlg.m_PeerAddr);
			m_PortNo = dlg.m_PortNo;
			m_UserName.SetString(dlg.m_UserName);

			if(m_SockMode==0) {

				// if the socket hasn't been created, make it
				if(!m_CliSock.GetSockHandle()) {
					if(!m_CliSock.Create()) {
						theAPI.ReportError("CChatDlg::OnBnClickedConnectButton error:\r\n - cannot create client socket");
						return;
					}
					if(!m_CliSock.SetCallback(SOCKCB_ONRECV, theAPI.zeofunc_GetFunc("zChatClient.ExtOnRecv"))) {
						theAPI.ReportError("CChatDlg::OnBnClickedConnectButton error:\r\n - cannot set OnRecv callback");
						return;
					}
					if(!m_CliSock.SetCallback(SOCKCB_ONCLOSE, theAPI.zeofunc_GetFunc("zChatClient.ExtOnClose"))) {
						theAPI.ReportError("CChatDlg::OnBnClickedConnectButton error:\r\n - cannot set OnClose callback");
						return;
					}
				}

				if(!m_CliSock.Connect(m_PeerAddr, m_PortNo)) {
					int code = m_CliSock.GetLastSockError();
					CString TempStr;
					TempStr.Format("Cannot connect to host '%s:%d' (error code = %s)", m_PeerAddr, m_PortNo, GetWinsockErrorStr(code));
					AfxMessageBox(TempStr, MB_ICONERROR);
				} else {
					if(m_UserName.GetLength()) {
						m_CliSock.SendMsgA("set display name", m_UserName);
					}
				}
			} else
			if(m_SockMode==1) {

				// if the socket hasn't been created, make it
				if(!m_ListenSock.GetSockHandle()) {
					if(!m_ListenSock.Create()) {
						theAPI.ReportError("CChatDlg::OnBnClickedConnectButton error:\r\n - cannot create listen socket");
						return;
					}
					if(!m_ListenSock.SetCallback(SOCKCB_ONACCEPT, theAPI.zeofunc_GetFunc("zChatClient.ExtOnAccept"))) {
						theAPI.ReportError("CChatDlg::OnBnClickedConnectButton error:\r\n - cannot set OnAccept callback");
						return;
					}
				}

				if(!m_ListenSock.Listen(m_PortNo)) {
					int code = m_ListenSock.GetLastSockError();
					CString TempStr;
					TempStr.Format("Cannot open port %d for listening (error code = %s)", m_PortNo, GetWinsockErrorStr(code));
					AfxMessageBox(TempStr, MB_ICONERROR);
				}
			}
		}
	}

	FormatControls();
}

void CChatDlg::OnBnClickedHelpButton() {
	ShellExecute(NULL, "Open", "http://www.bundysoft.com/wiki/doku.php?id=plugins:network:zChatClient", NULL, NULL, SW_NORMAL);
}


void CChatDlg::FormatControls() {

	CString TempStr;

	if(m_SockMode==0) { // client
		if(m_CliSock.GetSockState()==SOCKSTATE_CONNECTED) {
			m_ConnectBtnCtrl.SetWindowText("Connected");
			m_SendEditCtrl.EnableWindow(true);
			m_SendBtnCtrl.EnableWindow(true);

			TempStr.Format("Chat client - %s:%d", m_PeerAddr, m_PortNo);
		} else {
			m_ConnectBtnCtrl.SetWindowText("Closed");
			m_SendEditCtrl.EnableWindow(false);
			m_SendBtnCtrl.EnableWindow(false);

			TempStr.Format("Chat client - disconnected");
		}
	} else
	if(m_SockMode==1) { // server
		if(m_ListenSock.GetSockState()==SOCKSTATE_LISTENING) {
			m_ConnectBtnCtrl.SetWindowText("Listening");

			TempStr.Format("Chat host - Port %d", m_PortNo);

			if(m_PeerList.GetSize()) {
				m_SendEditCtrl.EnableWindow(true);
				m_SendBtnCtrl.EnableWindow(true);
			} else {
				m_SendEditCtrl.EnableWindow(false);
				m_SendBtnCtrl.EnableWindow(false);
			}
		} else {
			m_ConnectBtnCtrl.SetWindowText("Closed");
			m_SendEditCtrl.EnableWindow(false);
			m_SendBtnCtrl.EnableWindow(false);

			TempStr.Format("Chat host - closed");
		}
	}

	SetWindowText(TempStr);
}

void CChatDlg::OnConnect(LPVOID hSock) {
	return;
}
void CChatDlg::OnAccept(LPVOID hSock) {

	if(hSock!=m_ListenSock.GetSockHandle()) {
		theAPI.ReportError("CChatDlg::OnAccept error:\r\n - socket handle mismatch");
		return;
	}

	CzTcpSock* pSock = new CzTcpSock;
	if(!pSock->Create()) {
		theAPI.ReportError("CChatDlg::OnAccept error:\r\n - cannot create peer socket");
		delete pSock;
		return;
	}
	if(!pSock->SetCallback(SOCKCB_ONRECV, theAPI.zeofunc_GetFunc("zChatClient.ExtOnRecv"))) {
		theAPI.ReportError("CChatDlg::OnAccept error:\r\n - cannot set OnRecv callback");
		return;
	}
	if(!pSock->SetCallback(SOCKCB_ONCLOSE, theAPI.zeofunc_GetFunc("zChatClient.ExtOnClose"))) {
		theAPI.ReportError("CChatDlg::OnAccept error:\r\n - cannot set OnClose callback");
		return;
	}
	if(!m_ListenSock.Accept(pSock)) {
		theAPI.ReportError("CChatDlg::OnAccept error:\r\n - cannot accept peer connection");
		delete pSock;
		return;
	}

	m_PeerList.Add((CObject*)pSock);

	CzStr zPeerAddr;
	pSock->GetPeerAddr((ZVAR)zPeerAddr);

	CString TempStr;
	if(m_UserName.GetLength()) {
		TempStr.Format("Guest %s:%d has joined host '%s'", (char*)zPeerAddr, pSock->GetPeerPort(), m_UserName);
	} else {
		TempStr.Format("Guest %s:%d has joined host", (char*)zPeerAddr, pSock->GetPeerPort());
	}

	AddRecvText(TempStr);
	AddRecvText("\r\n\r\n");

	SendAllClients(TempStr);

	TempStr.Format("%d guest(s) are currently on-line", m_PeerList.GetSize());
	SendAllClients(TempStr);
	AddRecvText(TempStr);
	AddRecvText("\r\n\r\n");

	FormatControls();
}
void CChatDlg::OnRecv(LPVOID hSock) {

	UpdateData(TRUE);

	CzTcpSock zTempSock;
	if(!zTempSock.Attach(hSock)) {
		theAPI.ReportError("CChatDlg::OnRecv error:\r\n - invalid socket handle");
		return;
	}

	CzList zMsgQueue;
	if(!zTempSock.GetRecvQueue((ZLIST)zMsgQueue)) {
		theAPI.ReportError("CChatDlg::OnRecv error:\r\n - cannot get receive queue");
		return;
	}

	long nItems = zMsgQueue.nItems();
	for(long i=0; i<nItems; i++) {
		CzList Msg;
		if(!Msg.Attach(zMsgQueue.GetItem(i))) {
			theAPI.ReportError("CChatDlg::OnRecv error:\r\n - cannot get message");
		} else {
			CzStr Label;
			if(!Label.Attach(Msg.GetItem((int)0))) {
				theAPI.ReportError("CChatDlg::OnRecv error:\r\n - cannot get message label");
			} else {
				if(Label.CompareNoCase("text")==0) {
					CzBuffer Data;
					if(!Data.Attach(Msg.GetItem((int)1))) {
						theAPI.ReportError("CChatDlg::OnRecv error:\r\n - cannot get message data");
					} else {

						if(m_SockMode==0) {
							CString TempStr;
							TempStr.Format("%s\r\n\r\n", (char*)Data.GetAt(0));
							AddRecvText(TempStr);
						} else 
						if(m_SockMode==1) {

							// loopup peer
							CzTcpSock* pSock = NULL;
							bool DoneFlag = false;
							for(int i=0; i<m_PeerList.GetSize() && !DoneFlag; i++) {
								pSock = (CzTcpSock*)m_PeerList.GetAt(i);
									if(pSock->GetSockHandle()==hSock) {
										DoneFlag = true;
									}
							}
							
							CString TempStr;

							CzStr DisplayName;
							if(DisplayName.Attach(theAPI.list_GetItemA(pSock->m_hUserData, "DisplayName"))) {
								TempStr.Format("Guest '%s' says: \"%s\"", (char*)DisplayName, (char*)Data.GetAt(0));
							} else {
								CzStr zPeerAddr;
								unsigned long PeerPort = 0;
								if(DoneFlag && pSock) {
									pSock->GetPeerInfo((ZVAR)zPeerAddr, PeerPort);
								}
								TempStr.Format("Guest %s:%d says: \"%s\"", (char*)zPeerAddr, PeerPort, (char*)Data.GetAt(0));
							}
							
							SendAllClients(TempStr);

							AddRecvText(TempStr);
							AddRecvText("\r\n\r\n");
						}

						UpdateData(FALSE);
					}
				} else
				if(Label.CompareNoCase("set display name")==0) {
					CzBuffer Data;
					if(!Data.Attach(Msg.GetItem((int)1))) {
						theAPI.ReportError("CChatDlg::OnRecv error:\r\n - cannot get message data");
					} else {
						if(m_SockMode==1) {

							// loopup peer
							CzTcpSock* pSock = NULL;
							bool DoneFlag = false;
							for(int i=0; i<m_PeerList.GetSize() && !DoneFlag; i++) {
								pSock = (CzTcpSock*)m_PeerList.GetAt(i);
									if(pSock->GetSockHandle()==hSock) {
										DoneFlag = true;
									}
							}

							CzList SockData;
							SockData.Attach(pSock->m_hUserData);
							theAPI.str_SetText(SockData.GetOrCreateItem(VarID_string, "DisplayName"), (char*)Data.GetAt(0));

							CzStr zPeerAddr;
							unsigned long PeerPort = 0;
							if(DoneFlag && pSock) {
								pSock->GetPeerInfo((ZVAR)zPeerAddr, PeerPort);
							}
							CString TempStr;
							TempStr.Format("Guest %s:%d is '%s'", (char*)zPeerAddr, PeerPort, (char*)Data.GetAt(0));
						
							SendAllClients(TempStr);

							AddRecvText(TempStr);
							AddRecvText("\r\n\r\n");
						}

						UpdateData(FALSE);
					}
				}
			}
		}
	}

	return;
}
void CChatDlg::OnClose(LPVOID hSock) {

	UpdateData(TRUE);

	if(m_SockMode==0) { // client
		// do message (server has disconnected?)
		CString TempStr;
		TempStr.Format("Host has disconnected\r\n\r\n");
		AddRecvText(TempStr);
	} else
	if(m_SockMode==1) { // server

		// look for client socket to delete
		bool DoneFlag = false;
		CString TempStr;
		for(int i=0; i<m_PeerList.GetSize() && !DoneFlag; i++) {
			CzTcpSock* pSock = (CzTcpSock*)m_PeerList.GetAt(i);
			if(pSock->GetSockHandle()==hSock) {
				DoneFlag = true;

				CzStr zDispName;
				try {
					zDispName.Attach(theAPI.list_GetItemA(pSock->m_hUserData, "DisplayName"));
				} catch(...) {
					theAPI.ReportError("CChatDlg::OnClose exception:\r\n - cannot get display name");
				}

				try {

					if(zDispName.GetLength()) {
						TempStr.Format("Guest '%s' has disconnected", (char*)zDispName);
					} else {

						try {
							CzStr zPeerAddr;
							unsigned long PeerPort;
							pSock->GetPeerInfo((ZSTR)zPeerAddr, PeerPort);
							TempStr.Format("Guest %s:%d has disconnected", (char*)zPeerAddr, PeerPort);
						} catch(...) {
							theAPI.ReportError("CChatDlg::OnClose exception:\r\n - cannot get peer info");
						}
					}

					AddRecvText(TempStr);
					AddRecvText("\r\n\r\n");
				} catch(...) {
					theAPI.ReportError("CChatDlg::OnClose exception:\r\n - cannot get/display peer info");
				}

				try {
					m_PeerList.RemoveAt(i);
					pSock->Delete();
				} catch(...) {
					theAPI.ReportError("CChatDlg::OnClose exception:\r\n - cannot remove/delete peer info");
				}
			}
		}

		try {
			if(DoneFlag) {
				SendAllClients(TempStr);
			}

			TempStr.Format("%d guest(s) are currently on-line", m_PeerList.GetSize());
			SendAllClients(TempStr);
			AddRecvText(TempStr);
			AddRecvText("\r\n\r\n");
		} catch(...) {	
			theAPI.ReportError("CChatDlg::OnClose exception:\r\n - cannot update clients");
		}
	}

	UpdateData(FALSE);

	FormatControls();
}

void CChatDlg::CloseAll() {
	m_CliSock.Delete();
	m_ListenSock.Delete();

	CloseClients();
}
void CChatDlg::CloseClients() {

	for(;m_PeerList.GetSize();) {
		CzTcpSock* pSock = (CzTcpSock*)m_PeerList.GetAt(0);
		m_PeerList.RemoveAt(0, 1);
		delete pSock;
	}
}
void CChatDlg::OnBnClickedSendButton() {
	
	UpdateData(TRUE);

	if(!m_SendText.GetLength())
		return;

	if(m_SockMode==0) { // client
		m_CliSock.SendMsgA("text", m_SendText);
	} else
	if(m_SockMode==1) { // server
		CString TempStr;
		if(m_UserName.GetLength()) {
			TempStr.Format("Host '%s' says: \"%s\"", m_UserName, m_SendText);
		} else {
			TempStr.Format("Host says: \"%s\"", m_SendText);
		}
		SendAllClients(TempStr);

		AddRecvText(TempStr);
		AddRecvText("\r\n\r\n");
	}
	m_SendText = _T("");
	UpdateData(FALSE);
}

void CChatDlg::SendAllClients(const char* lpMessage) {
	for(int i=0; i<m_PeerList.GetSize(); i++) {
		CzTcpSock* pSock = (CzTcpSock*)m_PeerList.GetAt(i);
		if(pSock) {
			if(pSock->GetSockState()==SOCKSTATE_CONNECTED) {
				pSock->SendMsgA("text", lpMessage);
			}
		}
	}
}

void CChatDlg::AddRecvText(const char* lpText) {

	m_RecvText.Append(lpText);

	if(IsShown()) {
		UpdateData(FALSE);
		m_RecvEditCtrl.LineScroll(m_RecvEditCtrl.GetLineCount());
	}
}