#include "StdAfx.h"
#include "ztcpsock.h"

#include "..\plugin API\ExtAPI.h"
extern CExtAPI theAPI;

#include "..\plugin API\zFunc.h"
#include "..\plugin API\zVar.h"
#include "..\plugin API\zStr.h"
#include "..\plugin API\zList.h"
#include "..\plugin API\zBuffer.h"

// helper function
const char* GetWinsockErrorStr(long LastErrorCode) {
	switch(LastErrorCode) {
	case WSANOTINITIALISED:
		return _T("WSANOTINITIALISED");
		break;
	case WSAENETDOWN:
		return _T("WSAENETDOWN");
		break;
	case WSAEADDRINUSE:
		return _T("WSAEADDRINUSE");
		break;
	case WSAEFAULT:
		return _T("WSAEFAULT");
		break;
	case WSAEINPROGRESS:
		return _T("WSAEINPROGRESS");
		break;
	case WSAEINVAL:
		return _T("WSAEINVAL");
		break;
	case WSAENETRESET:
		return _T("WSAENETRESET");
		break;
	case WSAETIMEDOUT:
		return _T("WSAETIMEDOUT");
		break;
	case WSAENOPROTOOPT:
		return _T("WSAENOPROTOOPT");
		break;
	case WSAEISCONN:
		return _T("WSAEISCONN");
		break;
	case WSAENOTCONN:
		return _T("WSAENOTCONN");
		break;
	case WSAENOTSOCK:
		return _T("WSAENOTSOCK");
		break;
	case WSAEOPNOTSUPP:
		return _T("WSAEOPNOTSUPP");
		break;
	case WSAESHUTDOWN:
		return _T("WSAESHUTDOWN");
		break;
	case WSAEWOULDBLOCK:
		return _T("WSAEWOULDBLOCK");
		break;
	case WSAEMSGSIZE:
		return _T("WSAEMSGSIZE");
		break;
	case WSAECONNABORTED:
		return _T("WSAECONNABORTED");
		break;
	case WSAECONNRESET:
		return _T("WSAECONNRESET");
		break;
	case WSAEADDRNOTAVAIL:
		return _T("WSAEADDRNOTAVAIL");
		break;
	case WSAEAFNOSUPPORT:
		return _T("WSAEAFNOSUPPORT");
		break;
	case WSAECONNREFUSED:
		return _T("WSAECONNREFUSED");
		break;
	case WSAEDESTADDRREQ:
		return _T("WSAEDESTADDRREQ");
		break;
	case WSAEMFILE:
		return _T("WSAEMFILE");
		break;
	case WSAENETUNREACH:
		return _T("WSAENETUNREACH");
		break;
	case WSAENOBUFS:
		return _T("WSAENOBUFS");
		break;
	}
	
	return _T("unknown");
}


CzTcpSock::CzTcpSock(void) {
	m_hSock = NULL;
	m_hUserData = NULL;

	m_hGetRecvQueue = NULL;
	m_hGetRecvQueueSize = NULL;
	m_DeleteSockOnRelease = true;
}

CzTcpSock::~CzTcpSock(void) {
	Delete();
}

bool CzTcpSock::Create() {
	if(m_hSock) {
		// report error
		return false;
	}

	// create a new socket
	CzFunc f;
	if(!f.GetFunc("atTcpSock.new")) {
		theAPI.ReportError("CzTcpSock::Create error:\r\n - cannot get atTcpSock.new function handle");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::Create error:\r\n - atTcpSock.new function failed");
		return false;
	}
	if(!f.m_rval.GetValue(VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::Create error:\r\n - cannot get atTcpSock.new function return value");
		return false;
	}
	if(!m_hSock) {
		theAPI.ReportError("CzTcpSock::Create error:\r\n - invalid socket handle returned by atTcpSock.new function");
		return false;
	}

	m_hUserData = theAPI.var_CreateTemp(VarID_varlist);
	m_DeleteSockOnRelease = true;

	return true;
}
void CzTcpSock::Delete() {
	if(!m_hSock) {
		return;
	}

	if(m_DeleteSockOnRelease) {
		// delete an existing socket
		CzFunc f;
		if(!f.GetFunc("atTcpSock.delete")) {
			theAPI.ReportError("CzTcpSock::Delete error:\r\n - cannot get atTcpSock.delete function handle");
			return;
		}
		if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
			theAPI.ReportError("CzTcpSock::Delete error:\r\n - cannot set hSock arg for atTcpSock.delete function");
			return;
		}
		if(!f.Execute2()) {
			theAPI.ReportError("CzTcpSock::Delete error:\r\n - atTcpSock.delete function failed");
			return;
		}
	}

	m_hSock = NULL;

	if(m_hUserData) {
		theAPI.var_Delete(m_hUserData);
	}
}

bool CzTcpSock::Attach(LPVOID hSock) {
	if(!hSock) {
		return false;
	}

	m_hSock = hSock;

	if(m_hUserData) {
		// clear data list if it exists
		theAPI.list_DelAll(m_hUserData);
	} else {
		// .. else create data list
		m_hUserData = theAPI.var_CreateTemp(VarID_varlist);
	}

	m_DeleteSockOnRelease = false;

	return true;
}
void CzTcpSock::Detach() {
	m_hSock = NULL;

	if(m_hUserData) {
		theAPI.var_Delete(m_hUserData);
	}
}
LPVOID CzTcpSock::GetSockHandle() { return m_hSock; }

bool CzTcpSock::Connect(const char* lpPeerAddr, unsigned long PeerPort, unsigned long LocalPort) {

	if(!m_hSock) {
		return false;
	}

	// connect the socket
	CzFunc f;
	if(!f.GetFunc("atTcpSock.Connect")) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot get atTcpSock.Create function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot set hSock arg for atTcpSock.Create function");
		return false;
	}
	if(!theAPI.str_SetText(f.m_args.GetItem(1), lpPeerAddr)) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot set lpPeerAddr arg for atTcpSock.Create function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)2, VarID_uint, &PeerPort)) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot set PeerPort arg for atTcpSock.Create function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)3, VarID_uint, &LocalPort)) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot set LocalPort arg for atTcpSock.Create function");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - atTcpSock.Connect function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::Connect error:\r\n - cannot get atTcpSock.Connect function return value");
		return false;
	}
	
	return rval;
}

bool CzTcpSock::Listen(unsigned long LocalPort) {

	if(!m_hSock) {
		return false;
	}

	// connect the socket
	CzFunc f;
	if(!f.GetFunc("atTcpSock.Listen")) {
		theAPI.ReportError("CzTcpSock::Listen error:\r\n - cannot get atTcpSock.Listen function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::Listen error:\r\n - cannot set hSock arg for atTcpSock.Listen function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)1, VarID_uint, &LocalPort)) {
		theAPI.ReportError("CzTcpSock::Listen error:\r\n - cannot set LocalPort arg for atTcpSock.Listen function");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::Listen error:\r\n - atTcpSock.Listen function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::Listen error:\r\n - cannot get atTcpSock.Listen function return value");
		return false;
	}
	return rval;
}

bool CzTcpSock::Accept(CzTcpSock* pSock) {

	if(!m_hSock) {
		return false;
	}

	if(!pSock) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - invalid sokcet handle argument");
		return false;
	}

	if(GetSockState()!=SOCKSTATE_LISTENING) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - socket is not listening");
		return false;
	}
	LPVOID hSock2 = pSock->GetSockHandle();
	if(!hSock2) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - cannot het connection socket handle");
		return false;
	}
	if(pSock->GetSockState()!=SOCKSTATE_CLOSED) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - connection socket is open");
		return false;
	}	

	
	CzFunc f;
	if(!f.GetFunc("atTcpSock.Accept")) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - cannot get atTcpSock.Accept function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - cannot set hSock arg for atTcpSock.Accept function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)1, VarID_LPVOID, &hSock2)) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - cannot set hConnSock arg for atTcpSock.Accept function");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - atTcpSock.Accept function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::Accept error:\r\n - cannot get atTcpSock.Accept function return value");
		return false;
	}
	return rval;
}
void CzTcpSock::Close() {

	if(!m_hSock) {
		return;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.Close")) {
		theAPI.ReportError("CzTcpSock::Close error:\r\n - cannot get atTcpSock.Close function");
		return;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::Close error:\r\n - cannot set hSock arg for atTcpSock.Close function");
		return;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::Close error:\r\n - atTcpSock.Close function failed");
		return;
	}

	return;
}

long CzTcpSock::GetSockState() {

	if(!m_hSock) {
		return SOCKSTATE_ERROR;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetSockState")) {
		theAPI.ReportError("CzTcpSock::GetSockState error:\r\n - cannot get atTcpSock.GetSockState function");
		return SOCKSTATE_ERROR;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetSockState error:\r\n - cannot set hSock arg for atTcpSock.GetSockState function");
		return SOCKSTATE_ERROR;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetSockState error:\r\n - atTcpSock.GetSockState function failed");
		return SOCKSTATE_ERROR;
	}
	long rval = SOCKSTATE_CLOSED;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzTcpSock::GetSockState error:\r\n - cannot get atTcpSock.GetSockState function return value");
		return SOCKSTATE_ERROR;
	}

	return rval;
}

// for sending messages
bool CzTcpSock::SendMsg(const char* lpLabel, unsigned long nDataBytes, void* pData) {

	try {

		if(!m_hSock) {
			return false;
		}

		if(!lpLabel) {
			return false;
		}
		if(!strlen(lpLabel)) {
			return false;
		}
		if(nDataBytes && !pData) {
			return false;
		}

		CzFunc f;
		if(!f.GetFunc("atTcpSock.SendMsg")) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot get atTcpSock.SendMsg function");
			return false;
		}
		if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot set hSock arg for atTcpSock.SendMsg function");
			return false;
		}
		if(!theAPI.str_SetText(f.m_args.GetItem(1), lpLabel)) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot set lpLabel arg for atTcpSock.SendMsg function");
			return false;
		}
		if(!f.m_args.SetItemValue((int)2, VarID_uint, &nDataBytes)) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot set nDataBytes arg for atTcpSock.SendMsg function");
			return false;
		}
		if(!f.m_args.SetItemValue((int)3, VarID_LPVOID, &pData)) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot set pData arg for atTcpSock.SendMsg function");
			return false;
		}
		try {
			if(!f.Execute2()) {
				theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - atTcpSock.SendMsg function failed");
				return false;
			}
		} catch(...) {
			theAPI.ReportError("CzTcpSock::SendMsg exception:\r\n - exception thrown by atTcpSock.SendMsg");
			throw;
		}
		bool rval = false;
		if(!f.m_rval.GetValue(VarID_bool, &rval)) {
			theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - cannot get atTcpSock.SendMsg function return value");
			return false;
		}
		return rval;

	} catch(...) {
		theAPI.ReportError("CzTcpSock::SendMsg error:\r\n - atTcpSock.SendMsg exception!");
		return false;
	}
}
bool CzTcpSock::SendMsgA(const char* lpLabel, const char* lpMsgText) {

	if(!m_hSock) {
		return false;
	}

	if(!lpLabel) {
		return false;
	}
	if(!strlen(lpLabel)) {
		return false;
	}
	if(!lpMsgText) {
		return false;
	}
	if(!strlen(lpMsgText)) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.SendMsgA")) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - cannot get atTcpSock.SendMsgA function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - cannot set hSock arg for atTcpSock.SendMsgA function");
		return false;
	}
	if(!theAPI.str_SetText(f.m_args.GetItem(1), lpLabel)) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - cannot set lpLabel arg for atTcpSock.SendMsgA function");
		return false;
	}
	if(!theAPI.str_SetText(f.m_args.GetItem(2), lpMsgText)) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - cannot set lpMsgText arg for atTcpSock.SendMsgA function");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - atTcpSock.SendMsgA function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::SendMsgA error:\r\n - cannot get atTcpSock.SendMsgA function return value");
		return false;
	}
	return rval;
}

// callback setting
bool CzTcpSock::SetCallback(unsigned long CallbackID, ZFUNC hCallbackZFUNC) {

	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.SetCallback")) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - cannot get atTcpSock.SetCallback function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - cannot set hSock arg for atTcpSock.SetCallback function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)1, VarID_uint, &CallbackID)) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - cannot set CallbackID arg for atTcpSock.SetCallback function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)2, VarID_hVar, &hCallbackZFUNC)) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - cannot set hCallbackZFUNC arg for atTcpSock.SetCallback function");
		return false;
	}
	
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - atTcpSock.SetCallback function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::SetCallback error:\r\n - cannot get atTcpSock.SetCallback function return value");
		return false;
	}
	return rval;
}

// checking connection info
bool CzTcpSock::GetPeerAddr(ZSTR hAddrStr) {

	try {

		if(!m_hSock) {
			return false;
		}
		if(!hAddrStr) {
			return false;
		}

		CzFunc f;
		if(!f.GetFunc("atTcpSock.GetPeerAddr")) {
			theAPI.ReportError("CzTcpSock::GetPeerAddr error:\r\n - cannot get atTcpSock.GetPeerAddr function");
			return false;
		}
		if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
			theAPI.ReportError("CzTcpSock::GetPeerAddr error:\r\n - cannot set hSock arg for atTcpSock.GetPeerAddr function");
			return false;
		}
		if(!f.Execute2()) {
			theAPI.ReportError("CzTcpSock::GetPeerAddr error:\r\n - atTcpSock.GetPeerAddr function failed");
			return false;
		}
		if(!theAPI.var_CopyValue(hAddrStr, f.m_rval.GetZVAR())) {
			theAPI.ReportError("CzTcpSock::GetPeerAddr error:\r\n - cannot get atTcpSock.GetPeerAddr function return value");
			return false;
		}

		return true;

	} catch(...) {
		theAPI.ReportError("CzTcpSock::GetPeerAddr exception");
		return false;
	}
}
unsigned long CzTcpSock::GetPeerPort() {

	try {

		if(!m_hSock) {
			return 0;
		}

		CzFunc f;
		if(!f.GetFunc("atTcpSock.GetPeerPort")) {
			theAPI.ReportError("CzTcpSock::GetPeerPort error:\r\n - cannot get atTcpSock.GetPeerPort function");
			return 0;
		}
		if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
			theAPI.ReportError("CzTcpSock::GetPeerPort error:\r\n - cannot set hSock arg for atTcpSock.GetPeerPort function");
			return 0;
		}
		if(!f.Execute2()) {
			theAPI.ReportError("CzTcpSock::GetPeerPort error:\r\n - atTcpSock.GetPeerPort function failed");
			return 0;
		}
		unsigned long rval = 0;
		if(!f.m_rval.GetValue(VarID_uint, &rval)) {
			theAPI.ReportError("CzTcpSock::GetPeerPort error:\r\n - cannot get atTcpSock.GetPeerPort function return value");
			return 0;
		}

		return rval;
	} catch(...) {
		theAPI.ReportError("CzTcpSock::GetPeerPort exception");
		return false;
	}
}
unsigned long CzTcpSock::GetLocalPort() {

	try {

		if(!m_hSock) {
			return false;
		}

		CzFunc f;
		if(!f.GetFunc("atTcpSock.GetLocalPort")) {
			theAPI.ReportError("CzTcpSock::GetLocalPort error:\r\n - cannot get atTcpSock.GetLocalPort function");
			return 0;
		}
		if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
			theAPI.ReportError("CzTcpSock::GetLocalPort error:\r\n - cannot set hSock arg for atTcpSock.GetLocalPort function");
			return 0;
		}
		if(!f.Execute2()) {
			theAPI.ReportError("CzTcpSock::GetLocalPort error:\r\n - atTcpSock.GetLocalPort function failed");
			return 0;
		}
		unsigned long rval = 0;
		if(!f.m_rval.GetValue(VarID_uint, &rval)) {
			theAPI.ReportError("CzTcpSock::GetLocalPort error:\r\n - cannot get atTcpSock.GetLocalPort function return value");
			return 0;
		}

		return rval;

	} catch(...) {
		theAPI.ReportError("CzTcpSock::GetLocalPort exception");
		return false;
	}
}

bool CzTcpSock::GetPeerInfo(ZSTR hAddrStr, unsigned long& PeerPort) {

	if(!GetPeerAddr(hAddrStr)) {
		theAPI.ReportError("CzTcpSock::GetPeerInfo error:\r\n - cannot get peer address");
		return false;
	}
	PeerPort = GetPeerPort();

	return true;
}

long CzTcpSock::GetLastSockError() {

	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetLastError")) {
		theAPI.ReportError("CzTcpSock::GetLastError error:\r\n - cannot get atTcpSock.GetLastError function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetLastError error:\r\n - cannot set hSock arg for atTcpSock.GetLastError function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetLastError error:\r\n - atTcpSock.GetLastError function failed");
		return 0;
	}
	long rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzTcpSock::GetLastError error:\r\n - cannot get atTcpSock.GetLastError function return value");
		return 0;
	}

	return rval;
}

__int64 CzTcpSock::GetBytesRecv() {
	
	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetBytesRecv")) {
		theAPI.ReportError("CzTcpSock::GetBytesRecv error:\r\n - cannot get atTcpSock.GetBytesRecv function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetBytesRecv error:\r\n - cannot set hSock arg for atTcpSock.GetBytesRecv function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetBytesRecv error:\r\n - atTcpSock.GetBytesRecv function failed");
		return 0;
	}
	__int64 rval = 0;
	if(!f.m_rval.GetValue(VarID_int64, &rval)) {
		theAPI.ReportError("CzTcpSock::GetBytesRecv error:\r\n - cannot get atTcpSock.GetBytesRecv function return value");
		return 0;
	}

	return rval;
}
__int64 CzTcpSock::GetBytesSent() {
	
	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetBytesSent")) {
		theAPI.ReportError("CzTcpSock::GetBytesSent error:\r\n - cannot get atTcpSock.GetBytesSent function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetBytesSent error:\r\n - cannot set hSock arg for atTcpSock.GetBytesSent function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetBytesSent error:\r\n - atTcpSock.GetBytesSent function failed");
		return 0;
	}
	__int64 rval = 0;
	if(!f.m_rval.GetValue(VarID_int64, &rval)) {
		theAPI.ReportError("CzTcpSock::GetBytesSent error:\r\n - cannot get atTcpSock.GetBytesSent function return value");
		return 0;
	}

	return rval;
}
__int64 CzTcpSock::GetMsgsRecv() {
	
	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetMsgsRecv")) {
		theAPI.ReportError("CzTcpSock::GetMsgsRecv error:\r\n - cannot get atTcpSock.GetMsgsRecv function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetMsgsRecv error:\r\n - cannot set hSock arg for atTcpSock.GetMsgsRecv function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetMsgsRecv error:\r\n - atTcpSock.GetMsgsRecv function failed");
		return 0;
	}
	__int64 rval = 0;
	if(!f.m_rval.GetValue(VarID_int64, &rval)) {
		theAPI.ReportError("CzTcpSock::GetMsgsRecv error:\r\n - cannot get atTcpSock.GetMsgsRecv function return value");
		return 0;
	}

	return rval;
}
__int64 CzTcpSock::GetMsgsSent() {
	
	if(!m_hSock) {
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("atTcpSock.GetMsgsSent")) {
		theAPI.ReportError("CzTcpSock::GetMsgsSent error:\r\n - cannot get atTcpSock.GetMsgsSent function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetMsgsSent error:\r\n - cannot set hSock arg for atTcpSock.GetMsgsSent function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetMsgsSent error:\r\n - atTcpSock.GetMsgsSent function failed");
		return 0;
	}
	__int64 rval = 0;
	if(!f.m_rval.GetValue(VarID_int64, &rval)) {
		theAPI.ReportError("CzTcpSock::GetMsgsSent error:\r\n - cannot get atTcpSock.GetMsgsSent function return value");
		return 0;
	}

	return rval;
}

bool CzTcpSock::GetRecvQueue(ZLIST hRecvQueue) {
	
	if(!m_hSock) {
		return false;
	}

	if(!m_hGetRecvQueue) {
		if(!(m_hGetRecvQueue = theAPI.zeofunc_GetFunc("atTcpSock.CopyRecvQueue"))) {
			theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot get atTcpSock.CopyRecvQueue function");
			return false;
		}
	}

	CzFunc f;
	if(!f.Attach(m_hGetRecvQueue)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot attach atTcpSock.CopyRecvQueue function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot set hSock arg for atTcpSock.CopyRecvQueue function");
		return false;
	}
	if(!f.m_args.SetItemValue((int)1, VarID_hVar, &hRecvQueue)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot set hRecvQueue arg for atTcpSock.CopyRecvQueue function");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - atTcpSock.CopyRecvQueue function failed");
		return false;
	}
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot get atTcpSock.CopyRecvQueue function return value");
		return false;
	}

	return rval;
}

long CzTcpSock::GetRecvQueueSize() {
	if(!m_hSock) {
		return 0;
	}

	if(!m_hGetRecvQueueSize) {
		if(!(m_hGetRecvQueueSize = theAPI.zeofunc_GetFunc("atTcpSock.GetRecvQueueSize"))) {
			theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot get atTcpSock.GetRecvQueueSize function");
			return 0;
		}
	}

	CzFunc f;
	if(!f.Attach(m_hGetRecvQueueSize)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot attach atTcpSock.GetRecvQueueSize function");
		return 0;
	}
	if(!f.m_args.SetItemValue((int)0, VarID_LPVOID, &m_hSock)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot set hSock arg for atTcpSock.GetRecvQueueSize function");
		return 0;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - atTcpSock.GetRecvQueueSize function failed");
		return 0;
	}
	long rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzTcpSock::GetRecvQueue error:\r\n - cannot get atTcpSock.GetRecvQueueSize function return value");
		return false;
	}

	return rval;
}