#pragma once

#include "..\plugin API\ExtAPI_defines.h"

// socket-reading states
#define SOCKSTATE_ERROR		-1
#define SOCKSTATE_CLOSED	0
#define SOCKSTATE_LISTENING	1
#define SOCKSTATE_CONNECTED	3

// callback IDs
#define SOCKCB_ONRECV		1
#define SOCKCB_ONCONNECT	2
#define SOCKCB_ONACCEPT		3
#define SOCKCB_ONCLOSE		4

// helper function
const char* GetWinsockErrorStr(long LastErrorCode);

class CzTcpSock
{
public:
	CzTcpSock(void);
	~CzTcpSock(void);

	bool Create();
	void Delete();

	bool Attach(LPVOID hSock);
	void Detach();
	LPVOID GetSockHandle();

	bool Connect(const char* lpPeerAddr, unsigned long PeerPort, unsigned long LocalPort = 0);
	bool Listen(unsigned long LocalPort);
	bool Accept(CzTcpSock* pSock);
	void Close();
	
	long GetSockState();
	
	// for sending messages
	bool SendMsg(const char* lpLabel, unsigned long nDataBytes, void* pData);
	bool SendMsgA(const char* lpLabel, const char* lpMsgText);
	
	// callback setting
	bool SetCallback(unsigned long CallbackID, ZFUNC hCallbackZFUNC);
	
	// checking connection info
	bool GetPeerInfo(ZSTR hAddrStr, unsigned long& PeerPort);
	bool GetPeerAddr(ZSTR hAddrStr);
	unsigned long GetPeerPort();
	unsigned long GetLocalPort();

	long GetRecvQueueSize();
	bool GetRecvQueue(ZLIST hRecvQueue);

	__int64 GetBytesRecv();
	__int64 GetBytesSent();
	__int64 GetMsgsRecv();
	__int64 GetMsgsSent();

	long GetLastSockError();

	ZLIST m_hUserData;

protected:
	LPVOID m_hSock;

	bool m_DeleteSockOnRelease;

	ZFUNC m_hGetRecvQueue;
	ZFUNC m_hGetRecvQueueSize;
};
